#include "ir_rcv.h"
#include "print_lib.h"

#define MAX_COUNTER	100

#define TMR0VAL	0x72

int counter;

void interrupt intr()
{
	if (TMR0IF) {
		counter++;
		if (counter < MAX_COUNTER) {
			TMR0 = TMR0VAL;
			TMR0IF = 0;
			TMR0IE = 1;
		}
	}
}

void wait()
{
	for (int i = 0; i < 10; i++) {
		__delay_ms(100);
	}
}

void err_overflow()
{
	lcd_clear();
	lcd_str("OVERFLOW!");
	wait();
}

char check_reader()
{
	counter = 0;
	int readerH, readerL;
	OPTION = 0x81;	// prescale=4
	TMR0 = TMR0VAL;
	TMR0IF = 0;
	TMR0IE = 1;
	GIE = 1;

	while (IRRCV == 0) {
	}
	GIE = 0;
	readerH = counter;
	counter = 0;
	TMR0 = TMR0VAL;
	GIE = 1;
	while (IRRCV != 0) {
	}
	GIE = 0;
	readerL = counter;
	if (((readerH / readerL) == 2) && (readerL == 7)) {
		return READER_NEC;
	} else if (((readerH / readerL) >= 4) && (readerL == 3)) {
		return READER_NEC_REPEAT;
	} else {
#if 1
		lcd_clear();
		lcd_goto(0, 0);
		print_int("RH=", readerH);
		print_int(",RL=", readerL);
#endif
		return READER_UNKNOWN;
	}
}

void rcv_data_nec()
{
	int dataH, dataL;
	int bytes = 0, i = 0;
	char d = 0, data[4] = {0, 0, 0, 0};
	
	OPTION = 0x81;	// prescale=4
	TMR0IF = 0;
	TMR0IE = 1;

	while ((bytes > 3) || (counter < MAX_COUNTER)) {
		counter = 0;
		TMR0 = TMR0VAL;
		GIE = 1;

		WAIT_H;

		GIE = 0;
		dataH = counter;
		counter = 0;
		TMR0 = TMR0VAL;
		GIE = 1;

		WAIT_L;

		GIE = 0;
		dataL = counter;
		
		if (dataH <= 1) {
			if (dataL <= 1) {
				// data = 0
				d <<= 1;
				i++;
				if (i >= 8) {
					data[bytes++] = d;
					i = 0;
					data[bytes] = d = 0;
				}
			} else if (dataL <= 3) {
				// data = 1
				d <<= 1;
				d |= 0x01;
				i++;
				if (i >= 8) {
					data[bytes++] = d;
					i = 0;
					data[bytes] = d = 0;
				}
			} else if (dataL > 10) {
				break;
			} else {
				// error
				lcd_clear();
				lcd_str("ERROR 1:");
				print_int("i=", i);
				print_int(",b=", bytes);
				lcd_goto(1, 0);
				print_int("dataL=", dataL);
				wait();
				break;
			}
		} else {
			// error
			lcd_clear();
			lcd_str("ERROR 2:");
			print_int("i=", i);
			print_int(",b=", bytes);
			lcd_goto(1, 0);
			print_int("dataH=", dataH);
			wait();
			break;
		}
	}
	
	if ((bytes == 4) && (i == 0)) {
		lcd_clear();
		lcd_goto(0, 0);
		lcd_str("NEC*");
		if ((data[0] & data[1]) == 0) {
			lcd_str("|OK");
		} else {
			lcd_str("|NG");
		}
		if ((data[2] & data[3]) == 0) {
			lcd_str("|OK");
		} else {
			lcd_str("|NG");
		}
		lcd_goto(1, 0);
		lcd_str("D=");
		for (int j = 0; j < 4; j++) {
			print_hex_char(NULL, data[j]);
			if (j < 3) {
				lcd_str(":");
			}
		}
	} else {
		lcd_clear();
		lcd_goto(0, 0);
		lcd_str("NEC*");
		print_int(",b=", bytes);
		print_int(",i=", i);
		lcd_goto(1, 0);
		lcd_str("D=");
		for (int j = 0; (j < 4) && (j <= bytes); j++) {
			print_hex_char(NULL, data[j]);
		}
	}
	counter = 0;
}

void ir_rcv()
{
	char type;
	type = check_reader();
	if (type == READER_NEC) {
		rcv_data_nec();
		wait();
	} else if (type == READER_NEC_REPEAT) {
		lcd_goto(1, 0);
		lcd_str("NEC Repeat");
		wait();
		return;
	} else {
		lcd_goto(1, 0);
		lcd_str("UNKNOWN");
		wait();
		return;
	}
	for (int i = 0; i < 10; i++) {
		__delay_ms(100);
	}
}
