#include "lcd_lib.h"

void lcd_out(char code, char flag)
{
	char tmp = LCD_PORT & 0x0F;
	LCD_PORT = tmp | (0xF0 & code);
	
	if (flag == 0) {
		LCD_RS = 1;
	} else {
		LCD_RS = 0;		// コマンドデータの場合
	}
	__delay_us(1);		// pulse width
	LCD_E = 1;			// strobe out
	__delay_us(1);		// pulse width
	LCD_E = 0;			// reset strobe
}

void lcd_cmd(char cmd)
{
	lcd_out(cmd, 1);		// 上位4ビット出力
	lcd_out(cmd << 4, 1);	// 下位4ビット出力
	if ((cmd &0x03) != 0) {
		__delay_ms(2);		// 2msec待ち
	} else {
		__delay_us(50);		// 50μsec待ち
	}
}

void lcd_data(char data)
{
	lcd_out(data, 0);		// 上位4ビット出力
	lcd_out(data << 4, 0);	// 下位4ビット出力
	__delay_us(50);			// 50μsec待ち
}

// 行移動関数
void lcd_goto(unsigned char y, unsigned char x)
{
	char cmd;
	switch (y) {
	case 1:  cmd = 0xC0; break;
	case 2:  cmd = 0x94; break;
	case 3:  cmd = 0xD4; break;
	default: cmd = 0x80; break;
	}
	lcd_cmd(cmd+x);
}

// 全消去関数
void lcd_clear(void)
{
	lcd_cmd(0x01);			// 初期化コマンド出力
	__delay_ms(15);			// 15msec待ち
}

// 文字列出力関数
void lcd_str(const char *str)
{
	while (*str != 0x00) {
		lcd_data(*str);
		str++;
	}
}

void lcd_init()
{
	__delay_ms(30);
	__delay_ms(30);			// 30msec待ち
	lcd_cmd(0x03);			// 8bit mode set
	__delay_ms(5);
	lcd_cmd(0x03);			// 8bit mode set
	__delay_ms(1);
	lcd_cmd(0x03);			// 8bit mode set
	__delay_ms(1);
	lcd_cmd(0x02);			// 4bit mode set
	__delay_ms(1);
	lcd_cmd(0x28);			// DL=0 4bit mode, N=1 2lines, F=0 5x7 font
	__delay_ms(1);
	//lcd_cmd(0x08);			// display off D=C=B=0
	lcd_cmd(0x0C);			// display on D=1 C=B=0
	//lcd_cmd(0x06);			// entry I/D=1 S=0
	lcd_cmd(0x02);			// cursor home
	lcd_clear();
}
